class AudioManager {
    constructor() {
        this.audioContext = null;
        this.masterGainNode = null;
        this.isMuted = false;
        this.droneSound = null;
        this.droneSoundGain = null;
        this.initialized = false;
    }

    async init() {
        if (this.initialized) return;
        
        try {
            this.audioContext = new (window.AudioContext || window.webkitAudioContext)();
            this.masterGainNode = this.audioContext.createGain();
            this.masterGainNode.connect(this.audioContext.destination);
            this.initialized = true;
            await this.loadDroneSound();
        } catch (error) {
            console.warn('Audio initialization failed:', error);
        }
    }

    async loadDroneSound() {
        if (!this.audioContext) return;
        
        try {
            const response = await fetch('assets/drone-sound.mp3');
            const arrayBuffer = await response.arrayBuffer();
            const audioBuffer = await this.audioContext.decodeAudioData(arrayBuffer);
            
            // Create and configure drone sound
            this.droneSound = this.audioContext.createBufferSource();
            this.droneSound.buffer = audioBuffer;
            this.droneSound.loop = true;
            this.droneSound.playbackRate.value = 1.0;

            // Create gain node for drone sound
            this.droneSoundGain = this.audioContext.createGain();
            this.droneSoundGain.gain.value = 0;

            // Connect nodes
            this.droneSound.connect(this.droneSoundGain);
            this.droneSoundGain.connect(this.masterGainNode);
            
            // Start playing (it will be silent due to gain being 0)
            this.droneSound.start();
            console.log('Drone sound loaded and started');
        } catch (error) {
            console.warn('Could not load drone sound:', error);
        }
    }

    setMasterVolume(volume) {
        if (!this.masterGainNode) return;
        
        this.masterGainNode.gain.value = volume;
        this.isMuted = volume === 0;
        
        if (this.audioContext) {
            if (this.isMuted) {
                this.audioContext.suspend();
            } else {
                this.audioContext.resume();
            }
        }
    }

    updateDroneSound(speed, pitch = 1.0) {
        if (!this.droneSoundGain || !this.audioContext || this.isMuted) return;
        
        try {
            // Calculate volume based on speed (0 to 1)
            const normalizedSpeed = Math.min(Math.max(speed / 50, 0), 1);
            const targetVolume = normalizedSpeed * 0.8;

            // Smoothly update volume
            this.droneSoundGain.gain.setTargetAtTime(
                targetVolume,
                this.audioContext.currentTime,
                0.05
            );

            // Smoothly update playback rate for pitch
            if (this.droneSound && this.droneSound.playbackRate) {
                this.droneSound.playbackRate.setTargetAtTime(
                    pitch,
                    this.audioContext.currentTime,
                    0.1 // Slightly slower transition for pitch
                );
            }
        } catch (error) {
            console.warn('Error updating drone sound:', error);
        }
    }

    playBoostSound() {
        if (!this.audioContext || this.isMuted) return;

        const oscillator = this.audioContext.createOscillator();
        const gainNode = this.audioContext.createGain();
        
        oscillator.type = 'sine';
        oscillator.frequency.setValueAtTime(440, this.audioContext.currentTime);
        gainNode.gain.setValueAtTime(0.1, this.audioContext.currentTime);
        
        oscillator.connect(gainNode);
        gainNode.connect(this.masterGainNode);
        
        oscillator.start();
        oscillator.frequency.exponentialRampToValueAtTime(880, this.audioContext.currentTime + 0.2);
        gainNode.gain.exponentialRampToValueAtTime(0.01, this.audioContext.currentTime + 0.2);
        oscillator.stop(this.audioContext.currentTime + 0.2);
    }

    playCollisionSound() {
        if (!this.audioContext || this.isMuted) return;

        const oscillator = this.audioContext.createOscillator();
        const gainNode = this.audioContext.createGain();
        
        oscillator.type = 'square';
        oscillator.frequency.setValueAtTime(200, this.audioContext.currentTime);
        gainNode.gain.setValueAtTime(0.1, this.audioContext.currentTime);
        
        oscillator.connect(gainNode);
        gainNode.connect(this.masterGainNode);
        
        oscillator.start();
        gainNode.gain.exponentialRampToValueAtTime(0.01, this.audioContext.currentTime + 0.3);
        oscillator.stop(this.audioContext.currentTime + 0.3);
    }

    async resumeAudio() {
        if (!this.initialized) {
            await this.init();
        }
        
        if (this.audioContext) {
            try {
                await this.audioContext.resume();
                console.log('AudioContext resumed successfully');
            } catch (error) {
                console.warn('Error resuming AudioContext:', error);
            }
        }
    }
} 