class Drone {
    constructor(droneType = 'racing', scene, audioManager) {
        this.droneType = droneType;
        this.scene = scene;
        this.audioManager = audioManager;
        this.propellerOscillators = [];
        
        // Initialize basic properties
        this.velocity = new THREE.Vector3();
        this.acceleration = new THREE.Vector3();
        this.rotation = new THREE.Vector3();
        this.rotationVelocity = 0;
        this.thrust = 0;
        this.health = 100;
        this.isFlying = false;
        this.isLanding = false;
        this.isCrashed = false;
        this.isShaking = false;
        this.shakeTime = 0;
        this.crashes = 0;
        this.landings = 0;
        this.flightStartTime = null;
        this.totalFlightTime = 0;
        this.actualFlightTime = 0;
        this.boostActive = false;
        this.boostAvailable = true;
        this.boostCooldown = 0;
        this.boostStartTime = 0;
        this.damagePerCrash = 20;
        this.propellers = [];
        this.originalMaterials = new Map();
        this.drag = 0.99;
        this.rotationDrag = 0.95;
        this.maxHealth = 100;
        this.healthRecoveryRate = 2;
        this.lastDamageTime = 0;
        this.damageBlinkDuration = 2000;
        this.criticalHealthThreshold = 30;
        this.isBlinking = false;
        this.blinkInterval = null;
        this.boostFactor = 2.0;
        this.boostDuration = 3.0;
        this.boostTimer = 0;
        this.boostCooldownTimer = 0;
        
        // Create the drone mesh based on type
        if (droneType === 'racing') {
            this.mesh = this.createDroneMesh();
        } else {
            this.mesh = this.createClassicDroneMesh();
        }
        
        // Store original position and materials
        this.originalPosition = this.mesh.position.clone();
        this.mesh.traverse(child => {
            if (child.material) {
                this.originalMaterials.set(child, {
                    color: child.material.color.clone(),
                    emissive: child.material.emissive ? child.material.emissive.clone() : new THREE.Color(0x000000),
                    emissiveIntensity: child.material.emissiveIntensity || 0
                });
            }
        });
        
        // Initialize boost trail
        this.boostTrail = null;
        this.boostParticles = [];
        
        // Set initial position
        this.mesh.position.set(0, 5, 0);
        
        // Add stats tracking
        this.stats = {
            crashes: 0,
            landings: 0,
            flightStartTime: null,
            totalFlightTime: 0,
            isFlying: false,
            boostActive: false,
            boostCooldown: 0,
            boostAvailable: true
        };
    }

    playCollisionSound() {
        if (this.audioManager) {
            this.audioManager.playCollisionSound();
        }
    }

    playBoostSound() {
        if (this.audioManager) {
            this.audioManager.playBoostSound();
        }
    }

    createDroneMesh() {
        const group = new THREE.Group();

        // Materials
        const frameMaterial = new THREE.MeshPhongMaterial({
            color: 0x2a2a2a,
            specular: 0x111111,
            shininess: 100
        });

        const carbonFiberMaterial = new THREE.MeshPhongMaterial({
            color: 0x111111,
            specular: 0x222222,
            shininess: 30
        });

        // Main frame - X configuration
        const frameGeometry = new THREE.BoxGeometry(2.2, 0.15, 0.3);
        const frame1 = new THREE.Mesh(frameGeometry, frameMaterial);
        const frame2 = new THREE.Mesh(frameGeometry, frameMaterial);
        frame1.rotation.y = Math.PI / 4;
        frame2.rotation.y = -Math.PI / 4;
        group.add(frame1);
        group.add(frame2);

        // Center body - stack design
        const createStackLayer = (y, scale = 1) => {
            const bodyGeometry = new THREE.BoxGeometry(0.8 * scale, 0.08, 0.8 * scale);
            const body = new THREE.Mesh(bodyGeometry, carbonFiberMaterial);
            body.position.y = y;
            return body;
        };

        // Add stack layers
        group.add(createStackLayer(0, 1.2));    // Base plate
        group.add(createStackLayer(0.1, 1.1));  // ESC layer
        group.add(createStackLayer(0.2, 1.0));  // Flight controller
        group.add(createStackLayer(0.3, 0.9));  // VTX layer

        // FPV Camera mount
        const cameraMountGeometry = new THREE.BoxGeometry(0.4, 0.4, 0.2);
        const cameraMount = new THREE.Mesh(cameraMountGeometry, carbonFiberMaterial);
        cameraMount.position.set(0, 0.2, 0.4);
        cameraMount.rotation.x = Math.PI / 6; // Typical FPV camera angle
        group.add(cameraMount);

        // Camera lens
        const lensGeometry = new THREE.CylinderGeometry(0.15, 0.12, 0.2, 16);
        const lensMaterial = new THREE.MeshPhongMaterial({
            color: 0x111111,
            specular: 0x666666,
            shininess: 100
        });
        const lens = new THREE.Mesh(lensGeometry, lensMaterial);
        lens.rotation.x = Math.PI / 2;
        lens.position.set(0, 0.2, 0.5);
        lens.rotation.x = Math.PI / 6; // Match camera mount angle
        group.add(lens);

        // Motors and props at each arm end
        const motorPositions = [
            { x: 1.1, z: 1.1, ccw: true },    // Front Right
            { x: -1.1, z: 1.1, ccw: false },   // Front Left
            { x: -1.1, z: -1.1, ccw: true },   // Back Left
            { x: 1.1, z: -1.1, ccw: false }    // Back Right
        ];

        motorPositions.forEach(pos => {
            // Motor bell
            const motorGeometry = new THREE.CylinderGeometry(0.15, 0.18, 0.25, 12);
            const motor = new THREE.Mesh(motorGeometry, frameMaterial);
            motor.position.set(pos.x, 0.15, pos.z);
            group.add(motor);

            // Motor windings (detail)
            const windingGeometry = new THREE.TorusGeometry(0.16, 0.04, 8, 12);
            const windingMaterial = new THREE.MeshPhongMaterial({ color: 0xcc3300 });
            const winding = new THREE.Mesh(windingGeometry, windingMaterial);
            winding.position.set(pos.x, 0.15, pos.z);
            winding.rotation.x = Math.PI / 2;
            group.add(winding);

            // Propeller
            const propeller = new THREE.Group();
            const bladeGeo = new THREE.BoxGeometry(1.6, 0.04, 0.15);
            bladeGeo.translate(0.4, 0, 0);
            const bladeMat = new THREE.MeshPhongMaterial({
                color: pos.ccw ? 0xff3333 : 0x3333ff,
                transparent: true,
                opacity: 0.9
            });

            const blade1 = new THREE.Mesh(bladeGeo, bladeMat);
            const blade2 = new THREE.Mesh(bladeGeo, bladeMat);
            blade2.rotation.y = Math.PI;

            propeller.add(blade1);
            propeller.add(blade2);
            propeller.position.set(pos.x, 0.3, pos.z);
            
            group.add(propeller);
            this.propellers.push(propeller);
        });

        // Add LED strips
        const createLEDStrip = (x, z, color) => {
            const ledGeometry = new THREE.BoxGeometry(0.3, 0.02, 0.05);
            const ledMaterial = new THREE.MeshPhongMaterial({
                color: color,
                emissive: color,
                emissiveIntensity: 0.5
            });
            const led = new THREE.Mesh(ledGeometry, ledMaterial);
            led.position.set(x, 0.1, z);
            return led;
        };

        // Add LED strips on arms
        group.add(createLEDStrip(0.8, 0.8, 0x00ff00));   // Front Right
        group.add(createLEDStrip(-0.8, 0.8, 0x00ff00));  // Front Left
        group.add(createLEDStrip(-0.8, -0.8, 0xff0000)); // Back Left
        group.add(createLEDStrip(0.8, -0.8, 0xff0000));  // Back Right

        // Add battery strap
        const strapGeometry = new THREE.BoxGeometry(1.0, 0.05, 0.4);
        const strapMaterial = new THREE.MeshPhongMaterial({ color: 0x222222 });
        const strap = new THREE.Mesh(strapGeometry, strapMaterial);
        strap.position.y = 0.15;
        group.add(strap);

        // Scale the entire drone
        group.scale.set(0.52, 0.52, 0.52);

        return group;
    }

    createClassicDroneMesh() {
        const group = new THREE.Group();

        // Carbon fiber material
        const carbonFiberMaterial = new THREE.MeshPhongMaterial({
            color: 0x1a1a1a,
            specular: 0x333333,
            shininess: 100,
            roughness: 0.5
        });

        // Metallic silver material
        const metallicMaterial = new THREE.MeshPhongMaterial({
            color: 0x888888,
            specular: 0xffffff,
            shininess: 200,
            metalness: 0.8
        });

        // Main body - streamlined design
        const bodyShape = new THREE.Shape();
        bodyShape.moveTo(-0.8, -1);
        bodyShape.quadraticCurveTo(0, -1.2, 0.8, -1);
        bodyShape.lineTo(0.8, 1);
        bodyShape.quadraticCurveTo(0, 1.2, -0.8, 1);
        bodyShape.lineTo(-0.8, -1);

        const extrudeSettings = {
            steps: 1,
            depth: 0.3,
            bevelEnabled: true,
            bevelThickness: 0.1,
            bevelSize: 0.1,
            bevelSegments: 3
        };

        const bodyGeometry = new THREE.ExtrudeGeometry(bodyShape, extrudeSettings);
        const body = new THREE.Mesh(bodyGeometry, carbonFiberMaterial);
        body.rotation.x = Math.PI / 2;
        body.scale.set(1, 1, 0.5);
        this.originalMaterials.set(body, carbonFiberMaterial.clone());
        group.add(body);

        // Top cover with air vents
        const topCoverGeometry = new THREE.BoxGeometry(1.4, 0.05, 1.8);
        const topCover = new THREE.Mesh(topCoverGeometry, metallicMaterial);
        topCover.position.y = 0.25;
        this.originalMaterials.set(topCover, metallicMaterial.clone());
        group.add(topCover);

        // Air vents
        const ventGeometry = new THREE.BoxGeometry(0.4, 0.02, 0.05);
        const ventMaterial = new THREE.MeshPhongMaterial({ color: 0x111111 });
        for (let i = 0; i < 6; i++) {
            const vent = new THREE.Mesh(ventGeometry, ventMaterial);
            vent.position.set(0, 0.26, -0.6 + i * 0.2);
            this.originalMaterials.set(vent, ventMaterial.clone());
            group.add(vent);
        }

        // Create arms in X pattern
        const armPositions = [
            { x: 0.8, z: 0.8, color: 0x00ffff },   // Front Right
            { x: -0.8, z: 0.8, color: 0x00ffff },  // Front Left
            { x: -0.8, z: -0.8, color: 0x00ffff }, // Back Left
            { x: 0.8, z: -0.8, color: 0x00ffff }   // Back Right
        ];

        armPositions.forEach((pos, i) => {
            // Arm joint
            const jointGeometry = new THREE.SphereGeometry(0.12, 8, 8);
            const joint = new THREE.Mesh(jointGeometry, metallicMaterial);
            joint.position.set(pos.x * 0.5, 0, pos.z * 0.5);
            this.originalMaterials.set(joint, metallicMaterial.clone());
            group.add(joint);

            // Carbon fiber arm
            const armGeometry = new THREE.BoxGeometry(0.8, 0.08, 0.12);
            const arm = new THREE.Mesh(armGeometry, carbonFiberMaterial);
            arm.position.set(pos.x, 0, pos.z);
            this.originalMaterials.set(arm, carbonFiberMaterial.clone());
            group.add(arm);

            // Motor housing
            const motorHousingGeometry = new THREE.CylinderGeometry(0.15, 0.18, 0.25, 12);
            const motorHousing = new THREE.Mesh(motorHousingGeometry, metallicMaterial);
            motorHousing.position.set(pos.x, 0.15, pos.z);
            this.originalMaterials.set(motorHousing, metallicMaterial.clone());
            group.add(motorHousing);

            // LED ring
            const ledRingGeometry = new THREE.TorusGeometry(0.2, 0.02, 8, 24);
            const ledRingMaterial = new THREE.MeshPhongMaterial({
                color: pos.color,
                emissive: pos.color,
                emissiveIntensity: 0.5,
                transparent: true,
                opacity: 0.8
            });
            const ledRing = new THREE.Mesh(ledRingGeometry, ledRingMaterial);
            ledRing.position.set(pos.x, 0.3, pos.z);
            ledRing.rotation.x = Math.PI / 2;
            this.originalMaterials.set(ledRing, ledRingMaterial.clone());
            group.add(ledRing);

            // Create propeller
            const propeller = new THREE.Group();
            const bladeGeo = new THREE.BoxGeometry(1.2, 0.02, 0.12);
            bladeGeo.translate(0.3, 0, 0);
            const bladeMat = new THREE.MeshPhongMaterial({
                color: 0x333333,
                transparent: true,
                opacity: 0.9
            });

            const blade1 = new THREE.Mesh(bladeGeo, bladeMat);
            const blade2 = new THREE.Mesh(bladeGeo, bladeMat);
            blade2.rotation.y = Math.PI;

            this.originalMaterials.set(blade1, bladeMat.clone());
            this.originalMaterials.set(blade2, bladeMat.clone());

            propeller.add(blade1);
            propeller.add(blade2);
            propeller.position.set(pos.x, 0.35, pos.z);
            
            group.add(propeller);
            this.propellers.push(propeller);
        });

        // Scale the entire drone
        group.scale.set(0.52, 0.52, 0.52);

        return group;
    }

    resetMaterials() {
        this.mesh.traverse(child => {
            if (child.material) {
                const originalMaterial = this.originalMaterials.get(child);
                if (originalMaterial) {
                    // Reset color
                    child.material.color.copy(originalMaterial.color);
                    // Reset emissive properties
                    child.material.emissive.copy(originalMaterial.emissive);
                    child.material.emissiveIntensity = originalMaterial.emissiveIntensity;
                    // Reset opacity
                    child.material.opacity = 1;
                    child.material.transparent = originalMaterial.transparent || false;
                }
            }
        });
    }

    crash() {
        if (this.isCrashed) return;
        
        this.isCrashed = true;
        this.isFlying = false;
        this.stats.crashes++;
        
        // Update crash animation and effects
        this.startDamageEffect();
        this.playCollisionSound();
        
        // Reset after crash
        setTimeout(() => {
            this.isCrashed = false;
            this.resetMaterials();
        }, 1000);
    }

    land() {
        if (!this.isFlying) return;
        
        this.isFlying = false;
        this.stats.landings++;
        this.updateFlightTime();
    }

    takeoff() {
        if (this.isFlying) return;
        
        this.isFlying = true;
        this.stats.flightStartTime = Date.now();
    }

    activateBoost() {
        if (!this.stats.boostAvailable || this.stats.boostActive) return;
        
        // Play boost sound
        if (this.audioManager) {
            this.playBoostSound();
        }
        
        this.stats.boostActive = true;
        this.boostActive = true;  // Set both flags
        this.stats.boostAvailable = false;
        this.createBoostTrail();
        this.boostTimer = this.boostDuration; // Initialize boost timer
        
        // Visual effect on propellers
        this.propellers.forEach(propeller => {
            propeller.children.forEach(blade => {
                if (blade.material) {
                    blade.material.emissive = new THREE.Color(0x00ffff);
                    blade.material.emissiveIntensity = 0.5;
                }
            });
        });
    }

    updateFlightTime() {
        if (this.stats.flightStartTime) {
            this.stats.totalFlightTime += (Date.now() - this.stats.flightStartTime) / 1000;
            this.stats.flightStartTime = null;
        }
    }

    createBoostTrail() {
        // Create a particle system for the boost trail
        const particleCount = 100;
        const particles = new THREE.BufferGeometry();
        
        const positions = new Float32Array(particleCount * 3);
        const colors = new Float32Array(particleCount * 3);
        
        const color = new THREE.Color();
        
        for (let i = 0; i < particleCount; i++) {
            // Initial positions behind the drone
            positions[i * 3] = 0;
            positions[i * 3 + 1] = 0;
            positions[i * 3 + 2] = 0;
            
            // Blue to cyan color gradient
            const mixRatio = i / particleCount;
            color.setHSL(0.6 - mixRatio * 0.1, 1.0, 0.5 + mixRatio * 0.5);
            
            colors[i * 3] = color.r;
            colors[i * 3 + 1] = color.g;
            colors[i * 3 + 2] = color.b;
        }
        
        particles.setAttribute('position', new THREE.BufferAttribute(positions, 3));
        particles.setAttribute('color', new THREE.BufferAttribute(colors, 3));
        
        const particleMaterial = new THREE.PointsMaterial({
            size: 0.2,
            vertexColors: true,
            transparent: true,
            opacity: 0.8
        });
        
        this.boostTrail = new THREE.Points(particles, particleMaterial);
        this.mesh.add(this.boostTrail);
        this.boostTrail.position.set(0, 0, 2); // Position behind the drone
    }
    
    updateBoostTrail() {
        if (this.boostTrail && this.boostActive) {
            const positions = this.boostTrail.geometry.attributes.position.array;
            const count = positions.length / 3;
            
            // Move particles back
            for (let i = count - 1; i > 0; i--) {
                positions[i * 3] = positions[(i - 1) * 3];
                positions[i * 3 + 1] = positions[(i - 1) * 3 + 1];
                positions[i * 3 + 2] = positions[(i - 1) * 3 + 2];
            }
            
            // Add random spread to the first particle
            positions[0] = (Math.random() - 0.5) * 0.5;
            positions[1] = (Math.random() - 0.5) * 0.5;
            positions[2] = 2 + (Math.random() - 0.5) * 0.5;
            
            this.boostTrail.geometry.attributes.position.needsUpdate = true;
        }
    }
    
    removeBoostTrail() {
        if (this.boostTrail) {
            this.mesh.remove(this.boostTrail);
            this.boostTrail = null;
        }
    }
    
    updateBoost(deltaTime) {
        // Update boost timer if active
        if (this.boostActive) {
            this.boostTimer -= deltaTime;
            this.updateBoostTrail();
            
            if (this.boostTimer <= 0) {
                // Deactivate boost
                this.boostActive = false;
                this.stats.boostActive = false;
                this.boostCooldownTimer = 5; // 5 seconds cooldown
                
                // Remove visual effects
                this.propellers.forEach(propeller => {
                    propeller.children.forEach(blade => {
                        if (blade.material) {
                            blade.material.emissive = new THREE.Color(0x000000);
                            blade.material.emissiveIntensity = 0;
                        }
                    });
                });
                
                this.removeBoostTrail();
                console.log("Boost deactivated, cooldown started.");
            }
        } 
        // Update cooldown timer if not available
        else if (!this.stats.boostAvailable) {
            this.boostCooldownTimer -= deltaTime;
            
            if (this.boostCooldownTimer <= 0) {
                this.stats.boostAvailable = true;
                this.boostAvailable = true;
                console.log("Boost available again!");
            }
        }
    }

    takeDamage(amount) {
        this.health = Math.max(0, this.health - amount);
        this.lastDamageTime = Date.now();
        
        // Start blinking if health is critical
        if (this.health <= this.criticalHealthThreshold) {
            this.startCriticalHealthBlink();
        }
        
        // Game over if health reaches 0
        if (this.health <= 0) {
            this.gameOver();
        }
    }

    startDamageEffect() {
        this.mesh.traverse(child => {
            if (child.material) {
                child.material.originalColor = child.material.color.clone();
                child.material.color.setHex(0xff0000);
                child.material.emissive.setHex(0xff0000);
                child.material.emissiveIntensity = 0.5;
            }
        });
    }

    startCriticalHealthBlink() {
        if (!this.isBlinking) {
            this.isBlinking = true;
            let isRed = false;
            
            this.blinkInterval = setInterval(() => {
                if (this.health > this.criticalHealthThreshold) {
                    clearInterval(this.blinkInterval);
                    this.isBlinking = false;
                    this.resetMaterials();
                    return;
                }
                
                this.mesh.traverse(child => {
                    if (child.isMesh) {
                        if (isRed) {
                            child.material.emissive.setRGB(0.5, 0, 0);
                        } else {
                            child.material.emissive.setRGB(0, 0, 0);
                        }
                    }
                });
                
                isRed = !isRed;
            }, 500);
        }
    }

    gameOver() {
        this.isFlying = false;
        this.velocity.set(0, 0, 0);
        
        // Create game over message
        const message = document.createElement('div');
        message.style.position = 'fixed';
        message.style.top = '50%';
        message.style.left = '50%';
        message.style.transform = 'translate(-50%, -50%)';
        message.style.backgroundColor = 'rgba(255, 0, 0, 0.8)';
        message.style.padding = '20px';
        message.style.borderRadius = '10px';
        message.style.color = 'white';
        message.style.fontSize = '24px';
        message.textContent = 'Game Over - Drone Destroyed!';
        
        document.body.appendChild(message);
        
        // Auto-reset after 3 seconds
        setTimeout(() => {
            message.remove();
            this.reset();
        }, 3000);
    }

    update(deltaTime) {
        // Update position based on velocity
        this.velocity.x += this.acceleration.x * deltaTime;
        this.velocity.y += this.acceleration.y * deltaTime;

        // Apply drag (reduced during boost)
        const currentDrag = this.boostActive ? 0.995 : this.drag;
        this.velocity.x *= currentDrag;
        this.velocity.y *= currentDrag;
        this.rotationVelocity *= this.rotationDrag;

        // Update position
        this.originalPosition.copy(this.mesh.position);
        this.mesh.position.add(this.velocity.clone().multiplyScalar(deltaTime));

        // Update rotation
        this.mesh.rotation.x += this.rotation.x * deltaTime;
        this.mesh.rotation.y += this.rotation.y * deltaTime;
        this.mesh.rotation.z += this.rotation.z * deltaTime;

        // Calculate current speed for audio
        const currentSpeed = Math.sqrt(this.velocity.x * this.velocity.x + this.velocity.y * this.velocity.y);
        if (this.audioManager) {
            this.audioManager.updateDroneSound(currentSpeed * (this.boostActive ? 1.5 : 1.0));
        }

        // Reset acceleration
        this.acceleration = { x: 0, y: 0 };

        // Update flight status based on movement and position
        const speed = this.velocity.length();
        const altitude = this.mesh.position.y + 10; // Adjust for ground level
        const isMoving = speed > 0.1;

        // Update flight status
        if (this.isShaking) {
            this.updateFlightStatus('Crashed', 'rgba(255, 0, 0, 0.2)');
        } else if (this.isLanding) {
            // Don't update status during landing animation
        } else if (this.isLanded) {
            this.updateFlightStatus('Landed', 'rgba(136, 204, 255, 0.2)');
        } else if (this.isFlying) {
            if (speed < 1) {
                this.updateFlightStatus('Hovering', 'rgba(255, 255, 0, 0.2)');
            } else {
                this.updateFlightStatus('Flying', 'rgba(0, 255, 0, 0.2)');
            }
        }

        // Handle takeoff when thrust is applied
        if (this.thrust > 9.8 && (this.isLanded || !this.isFlying)) {
            this.takeoff();
        }

        // Update rotation only when flying
        if (this.isFlying) {
            this.mesh.rotation.x += this.rotation.x * deltaTime;
            this.mesh.rotation.y += this.rotation.y * deltaTime;
            this.mesh.rotation.z += this.rotation.z * deltaTime;
        }

        // Update propeller rotation based on thrust and flight state
        this.propellerSpeed = Math.abs(this.thrust) * 20 + (this.isFlying ? 30 : 0);
        this.propellers.forEach((propeller, index) => {
            propeller.rotation.y += this.propellerSpeed * deltaTime * (index % 2 ? -1 : 1);
        });

        // Apply physics only when flying
        if (this.isFlying) {
            // Store horizontal velocity components
            const horizontalVelocity = new THREE.Vector3(this.velocity.x, 0, this.velocity.z);
            
            // Apply gravity
            this.velocity.y -= 9.8 * deltaTime;
            
            // Apply thrust only to vertical velocity
            this.velocity.y += this.thrust * deltaTime;
            
            // Apply drag separately to horizontal and vertical components
            horizontalVelocity.multiplyScalar(0.99);
            this.velocity.y *= 0.99;
            
            // Restore horizontal velocity
            this.velocity.x = horizontalVelocity.x;
            this.velocity.z = horizontalVelocity.z;
        } else {
            // When not flying, gradually reduce velocity
            this.velocity.multiplyScalar(0.95);
        }

        // Apply boost if active
        if (this.boostActive) {
            // Get forward direction based on drone's rotation
            const forwardDir = new THREE.Vector3(0, 0, -1);
            forwardDir.applyQuaternion(this.mesh.quaternion);
            forwardDir.normalize();
            
            // Apply stronger boost in forward direction
            const boostStrength = 50.0; // Increased from previous value
            forwardDir.multiplyScalar(boostStrength * deltaTime);
            this.velocity.add(forwardDir);
            
            // Update boost trail
            this.updateBoostTrail();
        }

        // Update boost status
        this.updateBoost(deltaTime);

        // Update flight times
        if (this.isFlying) {
            this.actualFlightTime += deltaTime;
        }
        this.totalFlightTime = (Date.now() - this.flightStartTime) / 1000;

        // Health recovery when not recently damaged
        const now = Date.now();
        if (now - this.lastDamageTime > 5000 && this.health < this.maxHealth) {
            this.health = Math.min(this.maxHealth, this.health + this.healthRecoveryRate * deltaTime);
        }

        // Update HUD with health
        const healthElement = document.getElementById('droneHealth');
        if (healthElement) {
            healthElement.textContent = Math.ceil(this.health);
            healthElement.style.color = this.health <= this.criticalHealthThreshold ? '#ff0000' : '#88ccff';
        }

        // Update flight status display
        const statusElement = document.getElementById('flightStatus');
        if (statusElement) {
            let status = 'Landed';
            let color = 'rgba(136, 204, 255, 0.2)';
            
            if (this.isFlying) {
                if (this.velocity.length() > 0.1) {
                    status = 'Flying';
                    color = 'rgba(0, 255, 0, 0.2)';
                } else {
                    status = 'Hovering';
                    color = 'rgba(255, 255, 0, 0.2)';
                }
            } else if (this.isShaking) {
                status = 'Crashed';
                color = 'rgba(255, 0, 0, 0.2)';
            }
            
            statusElement.textContent = status;
            statusElement.style.backgroundColor = color;
        }

        // Update boost if active
        if (this.boostActive || !this.stats.boostAvailable) {
            this.updateBoost(deltaTime);
        }

        // Update flight time if flying
        if (this.isFlying) {
            this.updateFlightTime();
        }
    }

    getFlightStats() {
        // Calculate current flight time
        let currentFlightTime = this.stats.totalFlightTime;
        if (this.isFlying && this.stats.flightStartTime) {
            currentFlightTime += (Date.now() - this.stats.flightStartTime) / 1000;
        }

        // Calculate crash rate
        const totalAttempts = this.stats.crashes + this.stats.landings;
        const crashRate = totalAttempts > 0 ? (this.stats.crashes / totalAttempts * 100).toFixed(1) : '0.0';

        return {
            crashes: this.stats.crashes,
            landings: this.stats.landings,
            flightTime: Math.floor(currentFlightTime) + 's',
            crashRate: crashRate + '%',
            boostActive: this.stats.boostActive,
            boostAvailable: this.stats.boostAvailable,
            boostCooldown: this.stats.boostCooldown,
            status: this.getFlightStatus()
        };
    }

    getFlightStatus() {
        if (this.isCrashed) return 'Crashed';
        if (!this.isFlying) return 'Landed';
        if (this.stats.boostActive) return 'Boost';
        return 'Flying';
    }

    reset() {
        // Reset position and rotation
        this.mesh.position.set(0, 0.5, 0);
        this.mesh.rotation.set(0, 0, 0);
        this.rotation.set(0, 0, 0);
        
        // Reset physics
        this.velocity.set(0, 0, 0);
        this.angularVelocity.set(0, 0, 0);
        this.thrust = 0;
        
        // Reset state
        this.isLanded = true;
        this.isFlying = false;
        this.isShaking = false;
        this.shakeTime = 0;
        this.health = 100;
        this.crashes = 0;
        this.landings = 0;
        
        // Reset boost
        this.boostActive = false;
        this.boostAvailable = true;
        this.boostTimer = 0;
        this.boostCooldownTimer = 0;
        
        // Reset materials and effects
        this.resetMaterials();
        if (this.boostTrail) {
            this.removeBoostTrail();
        }
        
        // Reset flight status
        this.updateFlightStatus('Landed', 'rgba(136, 204, 255, 0.2)');
    }

    updateFlightStatus(status, color) {
        // Update all flight status elements
        const statusElements = document.querySelectorAll('[id="flightStatus"]');
        statusElements.forEach(element => {
            if (element) {
                element.textContent = status;
                element.style.backgroundColor = color;
                
                // Add visual feedback based on status
                switch (status) {
                    case 'Landed':
                        element.style.color = '#ffffff';
                        break;
                    case 'Flying':
                        element.style.color = '#00ff00';
                        break;
                    case 'Hovering':
                        element.style.color = '#ffff00';
                        break;
                    case 'Crashed':
                        element.style.color = '#ff0000';
                        break;
                }
            }
        });
    }
}