class HUDPanel {
    constructor() {
        this.panel = document.createElement('div');
        this.panel.id = 'hudPanel';
        this.panel.style.cssText = `
            position: absolute;
            top: 20px;
            left: 20px;
            background-color: rgba(0, 0, 0, 0.7);
            color: #fff;
            padding: 12px;
            border-radius: 10px;
            font-family: 'Arial', sans-serif;
            width: 220px;
            z-index: 1000;
            backdrop-filter: blur(10px);
            border: 1px solid rgba(255, 255, 255, 0.1);
            font-size: 12px;
            display: none;
        `;
        
        // Track previous values for animations and updates
        this.previousValues = {
            score: 0,
            crashes: 0,
            landings: 0,
            health: 100,
            speed: 0
        };

        // Store element references
        this.elements = {};
        
        this.createStatElements();
        document.body.appendChild(this.panel);
    }

    createStatElements() {
        // Create sections for better organization - more minimalistic approach
        const sections = [
            {
                title: 'Flight Info',
                stats: [
                    { id: 'flightStatus', label: 'Status' },
                    { id: 'speed', label: 'Speed', unit: 'm/s' },
                    { id: 'altitude', label: 'Altitude', unit: 'm' },
                    { id: 'health', label: 'Health', unit: '%' },
                    { id: 'thrust', label: 'Thrust', unit: '%' },
                    { id: 'flightTime', label: 'Flight Time', unit: 's' }
                ]
            },
            {
                title: 'Position',
                stats: [
                    { id: 'posX', label: 'X-Axis' },
                    { id: 'posY', label: 'Y-Axis' },
                    { id: 'posZ', label: 'Z-Axis' },
                    { id: 'rotX', label: 'Pitch' },
                    { id: 'rotZ', label: 'Roll' }
                ]
            },
            {
                title: 'Mission',
                stats: [
                    { id: 'score', label: 'Score' },
                    { id: 'boostStatus', label: 'Boost' },
                    { id: 'crashes', label: 'Crashes' },
                    { id: 'landings', label: 'Safe Landings' }
                ]
            },
            {
                title: 'System',
                stats: [
                    { id: 'fps', label: 'FPS' },
                    { id: 'currentEnvironment', label: 'Environment' }
                ]
            }
        ];

        sections.forEach(section => {
            const sectionDiv = document.createElement('div');
            sectionDiv.style.cssText = `
                margin-bottom: 15px;
                padding-bottom: 10px;
                border-bottom: 1px solid rgba(255, 255, 255, 0.1);
            `;

            const title = document.createElement('h3');
            title.textContent = section.title;
            title.style.cssText = `
                color: rgba(255, 255, 255, 0.6);
                font-size: 12px;
                text-transform: uppercase;
                letter-spacing: 0.5px;
                margin: 0 0 8px 0;
            `;
            sectionDiv.appendChild(title);

            section.stats.forEach(stat => {
                const container = document.createElement('div');
                container.style.cssText = `
                    margin-bottom: 8px;
                    display: flex;
                    justify-content: space-between;
                    align-items: center;
                `;
                
                const label = document.createElement('span');
                label.textContent = `${stat.label}:`;
                label.style.cssText = `
                    color: #aaa;
                    margin-right: 8px;
                    font-size: 12px;
                `;
                
                const value = document.createElement('span');
                value.style.cssText = `
                    color: #fff;
                    font-weight: 500;
                    transition: all 0.2s ease;
                    font-size: 13px;
                `;
                
                if (stat.id === 'score') {
                    value.style.color = '#88ccff';
                    value.style.fontSize = '16px';
                }
                
                // Store element reference
                this.elements[stat.id] = value;
                
                container.appendChild(label);
                container.appendChild(value);
                sectionDiv.appendChild(container);
            });

            this.panel.appendChild(sectionDiv);
        });
    }

    update(stats) {
        // Ensure stats object exists
        stats = stats || {};
        
        // Update flight status
        if (this.elements.flightStatus) {
            const status = stats.status || 'Ready';
            this.elements.flightStatus.textContent = status;
            
            // Status color coding
            const statusColors = {
                'Flying': '#00ff00',
                'Crashed': '#ff0000',
                'Landed': '#88ccff',
                'Boost': '#ffff00',
                'Ready': '#ffffff'
            };
            this.elements.flightStatus.style.color = statusColors[status] || '#ffffff';
        }

        // Update score with animation
        if (this.elements.score) {
            const newScore = parseInt(stats.score) || 0;
            if (newScore !== this.previousValues.score) {
                this.elements.score.style.transform = 'scale(1.2)';
                this.elements.score.style.color = '#00ff00';
                this.elements.score.textContent = newScore;
                setTimeout(() => {
                    this.elements.score.style.transform = 'scale(1)';
                    this.elements.score.style.color = '#88ccff';
                }, 200);
                this.previousValues.score = newScore;
            }
        }

        // Update flight time
        if (this.elements.flightTime) {
            const totalSeconds = Math.floor(stats.flightTime || 0);
            const minutes = Math.floor(totalSeconds / 60);
            const seconds = totalSeconds % 60;
            this.elements.flightTime.textContent = `${minutes}:${seconds.toString().padStart(2, '0')}`;
        }

        // Update speed and altitude
        if (this.elements.speed) {
            const speed = parseFloat(stats.speed || 0).toFixed(1);
            this.elements.speed.textContent = `${speed} m/s`;
            if (speed > this.previousValues.speed) {
                this.elements.speed.style.color = '#00ff00';
            } else if (speed < this.previousValues.speed) {
                this.elements.speed.style.color = '#ff6666';
            }
            setTimeout(() => this.elements.speed.style.color = '#fff', 200);
            this.previousValues.speed = speed;
        }

        if (this.elements.altitude) {
            this.elements.altitude.textContent = `${parseFloat(stats.altitude || 0).toFixed(1)} m`;
        }

        // Update health with color coding
        if (this.elements.health) {
            const health = parseInt(stats.health || 100);
            this.elements.health.textContent = `${health}%`;
            if (health <= 25) {
                this.elements.health.style.color = '#ff0000';
            } else if (health <= 50) {
                this.elements.health.style.color = '#ff9900';
            } else {
                this.elements.health.style.color = '#00ff00';
            }
        }

        // Update thrust
        if (this.elements.thrust) {
            this.elements.thrust.textContent = `${parseInt(stats.thrust || 0)}%`;
        }

        // Update crashes and landings with animations
        const crashes = parseInt(stats.crashes) || 0;
        const landings = parseInt(stats.landings) || 0;
        
        if (this.elements.crashes) {
            if (crashes !== this.previousValues.crashes) {
                this.elements.crashes.style.transform = 'scale(1.2)';
                this.elements.crashes.style.color = '#ff6666';
                setTimeout(() => {
                    this.elements.crashes.style.transform = 'scale(1)';
                    this.elements.crashes.style.color = '#fff';
                }, 200);
                this.previousValues.crashes = crashes;
            }
            this.elements.crashes.textContent = crashes;
        }

        if (this.elements.landings) {
            if (landings !== this.previousValues.landings) {
                this.elements.landings.style.transform = 'scale(1.2)';
                this.elements.landings.style.color = '#00ff00';
                setTimeout(() => {
                    this.elements.landings.style.transform = 'scale(1)';
                    this.elements.landings.style.color = '#fff';
                }, 200);
                this.previousValues.landings = landings;
            }
            this.elements.landings.textContent = landings;
        }

        // Update boost status with animations
        if (this.elements.boostStatus) {
            let boostText, boostColor;
            
            if (stats.boostActive) {
                boostText = '🚀 ACTIVE';
                boostColor = '#ffff00';
            } else if (!stats.boostAvailable) {
                const cooldown = Math.ceil(stats.boostCooldown || 0);
                boostText = `⏳ COOLDOWN (${cooldown}s)`;
                boostColor = '#ff6666';
            } else {
                boostText = '✅ READY';
                boostColor = '#00ff00';
            }
            
            if (this.elements.boostStatus.textContent !== boostText) {
                this.elements.boostStatus.style.transform = 'scale(1.1)';
                setTimeout(() => this.elements.boostStatus.style.transform = 'scale(1)', 200);
            }
            
            this.elements.boostStatus.textContent = boostText;
            this.elements.boostStatus.style.color = boostColor;
        }

        // Update position and rotation
        if (this.elements.posX) this.elements.posX.textContent = parseFloat(stats.posX || 0).toFixed(1);
        if (this.elements.posY) this.elements.posY.textContent = parseFloat(stats.posY || 0).toFixed(1);
        if (this.elements.posZ) this.elements.posZ.textContent = parseFloat(stats.posZ || 0).toFixed(1);
        if (this.elements.rotX) {
            const pitchDeg = (parseFloat(stats.rotX || 0) * 180 / Math.PI).toFixed(1);
            this.elements.rotX.textContent = `${pitchDeg}°`;
            this.elements.rotX.style.color = Math.abs(pitchDeg) > 45 ? '#ff6666' : '#fff';
        }
        if (this.elements.rotZ) {
            const rollDeg = (parseFloat(stats.rotZ || 0) * 180 / Math.PI).toFixed(1);
            this.elements.rotZ.textContent = `${rollDeg}°`;
            this.elements.rotZ.style.color = Math.abs(rollDeg) > 45 ? '#ff6666' : '#fff';
        }

        // Update performance metrics
        if (this.elements.fps) this.elements.fps.textContent = stats.fps || 0;
        if (this.elements.frameTime) this.elements.frameTime.textContent = `${stats.frameTime || 0} ms`;

        // Update environment
        if (this.elements.currentEnvironment) {
            this.elements.currentEnvironment.textContent = stats.environment || 'City';
        }
    }
} 